<?php
/**
 * webEdition CMS
 *
 * $Rev: 8221 $
 * $Author: mokraemer $
 * $Date: 2014-09-11 22:38:50 +0200 (Thu, 11 Sep 2014) $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @category   webEdition
 * @package none
 * @license    http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * Util Functions
 *
 * all functions in this class are static! Please use it in static form:
 *    we_base_util::function_name();
 *
 *
 * @static
 */
abstract class we_base_util{
	const MIME_BY_HEAD_THEN_EXTENSION = 0;
	const MIME_BY_EXTENSION = 1;
	const MIME_BY_HEAD = 2;
	const MIME_BY_DATA = 3;

	//FIXME: add more extensions
	//NOTICE: WE contenttypes differ strongly from the following mime types!
	private static $mimetypes = array(
		'hqx' => 'application/mac-binhex40',
		'cpt' => 'application/mac-compactpro',
		'doc' => 'application/msword',
		'bin' => 'application/macbinary',
		'dms' => 'application/octet-stream',
		'lha' => 'application/octet-stream',
		'lzh' => 'application/octet-stream',
		'exe' => 'application/octet-stream',
		'class' => 'application/octet-stream',
		'psd' => 'application/octet-stream',
		'so' => 'application/octet-stream',
		'sea' => 'application/octet-stream',
		'dll' => 'application/octet-stream',
		'oda' => 'application/oda',
		'pdf' => 'application/pdf',
		'ai' => 'application/postscript',
		'eps' => 'application/postscript',
		'ps' => 'application/postscript',
		'smi' => 'application/smil',
		'smil' => 'application/smil',
		'mif' => 'application/vnd.mif',
		'xls' => 'application/vnd.ms-excel',
		'ppt' => 'application/vnd.ms-powerpoint',
		'wbxml' => 'application/vnd.wap.wbxml',
		'wmlc' => 'application/vnd.wap.wmlc',
		'dcr' => 'application/x-director',
		'dir' => 'application/x-director',
		'dxr' => 'application/x-director',
		'dvi' => 'application/x-dvi',
		'gtar' => 'application/x-gtar',
		'php' => 'application/x-httpd-php', //in WE this is text/html?
		'php4' => 'application/x-httpd-php',
		'php3' => 'application/x-httpd-php',
		'phtml' => 'application/x-httpd-php',
		'phps' => 'application/x-httpd-php-source',
		'js' => 'application/x-javascript',
		'swf' => 'application/x-shockwave-flash',
		'sit' => 'application/x-stuffit',
		'tar' => 'application/x-tar',
		'tgz' => 'application/x-tar',
		'xhtml' => 'application/xhtml+xml',
		'xht' => 'application/xhtml+xml',
		'zip' => 'application/zip',
		'mid' => 'audio/midi',
		'midi' => 'audio/midi',
		'mpga' => 'audio/mpeg',
		'mp2' => 'audio/mpeg',
		'mp3' => 'audio/mpeg',
		'aif' => 'audio/x-aiff',
		'aiff' => 'audio/x-aiff',
		'aifc' => 'audio/x-aiff',
		'ram' => 'audio/x-pn-realaudio',
		'rm' => 'audio/x-pn-realaudio',
		'rpm' => 'audio/x-pn-realaudio-plugin',
		'ra' => 'audio/x-realaudio',
		'rv' => 'video/vnd.rn-realvideo',
		'wav' => 'audio/x-wav',
		'bmp' => 'image/bmp',
		'gif' => 'image/gif',
		'jpeg' => 'image/jpeg',
		'jpg' => 'image/jpeg',
		'jpe' => 'image/jpeg',
		'png' => 'image/png',
		'tiff' => 'image/tiff',
		'tif' => 'image/tiff',
		'css' => 'text/css',
		'html' => 'text/html',
		'htm' => 'text/html',
		'shtml' => 'text/html',
		'txt' => 'text/plain',
		'text' => 'text/plain',
		'log' => 'text/plain',
		'rtx' => 'text/richtext',
		'rtf' => 'text/rtf',
		'xml' => 'text/xml',
		'xsl' => 'text/xml',
		'mpeg' => 'video/mpeg',
		'mpg' => 'video/mpeg',
		'mpe' => 'video/mpeg',
		'qt' => 'video/quicktime',
		'mov' => 'video/quicktime',
		'avi' => 'video/x-msvideo',
		'movie' => 'video/x-sgi-movie',
		'doc' => 'application/msword',
		'word' => 'application/msword',
		'xl' => 'application/excel',
		'eml' => 'message/rfc822',
		'svg' => 'image/svg+xml',
		'svgz' => 'image/svg+xml',
		'shtm' => 'text/html'
	);

	/**
	 * Formates a number with a country specific format into computer readable format.
	 * Returns the formated number.
	 *
	 * @static
	 * @access public
	 *
	 * @param mixed number
	 */
	static function std_numberformat($number){
		if(strpos($number, 'E')){ //  when number is too big, it is shown with E+xx
			$number = number_format($number, 2, '.', '');
		}
		$match = array();
		if(preg_match('|([0-9]*\.?[0-9]*),([0-9]*)|', $number, $match)){ // deutsche schreibweise
			return floatval(str_replace('.', '', $match[1]) . '.' . $match[2]);
		}
		if(preg_match('|([0-9]*)\.([0-9]*)|', $number)){ // engl schreibweise
			return floatval($number);
		}
		return floatval(str_replace(array(',', '.'), '', $number));
	}

	/**
	 * Converts all windows and mac newlines from string to unix newlines
	 * Returns the converted String.
	 *
	 * @static
	 * @access public
	 *
	 * @param mixed number
	 */
	static function cleanNewLine($string){
		return str_replace(array("\n\r", "\r\n", "\r"), "\n", $string);
	}

	/**
	 * Removes from string all newlines and converts all <br/> to newlines
	 * Returns the converted String.
	 *
	 * @static
	 * @access public
	 *
	 * @param mixed number
	 */
	static function br2nl($string){
		$string = str_replace(array("\n", "\r"), '', $string);
		return preg_replace('|<br ?/?>|i', "\n", $string);
	}

	static function rmPhp($in){
		$out = '';
		$starttag = strpos($in, '<?');
		if($starttag === false){
			return $in;
		}
		$lastStart = 0;
		while(!($starttag === false)){
			$endtag = strpos($in, '?>', $starttag);
			$out .= substr($in, $lastStart, ($starttag - $lastStart));
			$lastStart = $endtag + 2;
			$starttag = strpos($in, '<?', $lastStart);
		}
		if($lastStart < strlen($in)){
			$out .= substr($in, $lastStart, (strlen($in) - $lastStart));
		}
		return $out;
	}

	static function getGlobalPath(){
		return (isset($GLOBALS['WE_MAIN_DOC']) && isset($GLOBALS['WE_MAIN_DOC']->Path) ? $GLOBALS['WE_MAIN_DOC']->Path : '');
	}

	static function html2uml($text){
		return html_entity_decode($text, ENT_COMPAT, (isset($GLOBALS['CHARSET']) && $GLOBALS['CHARSET'] ? $GLOBALS['CHARSET'] : DEFAULT_CHARSET));
	}

	/**
	 * returns array of directory IDs of all directories which are located inside $folderID (recursive)
	 *
	 * @return array
	 * @param int $folderID
	 */
	static function getFoldersInFolder($folderID, $table = FILE_TABLE, we_database_base $db = null){
		$outArray = array(
			$folderID
		);
		$db = ($db ? $db : new DB_WE());
		$db->query('SELECT ID FROM ' . $table . ' WHERE ParentID=' . intval($folderID) . ' AND IsFolder=1');
		$new = array();
		while($db->next_record()){
			$new[] = $db->f('ID');
		}
		foreach($new as $cur){
			$tmpArray = self::getFoldersInFolder($cur, $table, $db);
			$outArray = array_merge($outArray, $tmpArray);
		}
		return $outArray;
	}

	/**
	 * Converts a given number in a via array specified system.
	 * as default a number is converted in the matching chars 0->^,1->a,2->b, ...
	 * other systems can simply set via the parameter $chars for example -> array(0,1)
	 * for bin-system
	 *
	 * @return string
	 * @param int $value
	 * @param array[optional] $chars
	 * @param string[optional] $str
	 */
	public static function number2System($value, $chars = array(), $str = ''){

		if(!(is_array($chars) && count($chars) > 1)){ //	in case of error take default-array
			$chars = array('^', 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z');
		}
		$base = count($chars);

//	get some information about the numbers:
		$_rest = $value % $base;
		$_result = ($value - $_rest) / $base;

//	1. Deal with the rest
		$str = $chars[$_rest] . $str;

//	2. Deal with remaining result
		return ($_result > 0 ? self::number2System($_result, $chars, $str) : $str);
	}

	static function getCurlHttp($server, $path, $files = array(), $header = false, $timeout = 0){
		$_response = array(
			'data' => '', // data if successful
			'status' => 0, // 0=ok otherwise error
			'error' => '' // error string
		);
		$parsedurl = parse_url($server);
		$protocol = (isset($parsedurl['scheme']) ?
				$parsedurl['scheme'] . '://' :
				'http://');

		$port = (isset($parsedurl['port']) ? ':' . $parsedurl['port'] : '');
		$_pathA = explode('?', $path);
		$_url = $protocol . $parsedurl['host'] . $port . $_pathA[0];
		if(isset($_pathA[1]) && strlen($_url . $_pathA[1]) < 2000){
//it is safe to have uri's lower than 2k chars - so no need to do a post which servers (e.g. twitter) do not accept.
			$_url.='?' . $_pathA[1];
			unset($_pathA[1]);
		}
		$_params = array();

		$_session = curl_init();
		curl_setopt($_session, CURLOPT_URL, $_url);
		curl_setopt($_session, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($_session, CURLOPT_FOLLOWLOCATION, true);
		curl_setopt($_session, CURLOPT_MAXREDIRS, 5);

		if($timeout){
			curl_setopt($_session, CURLOPT_CONNECTTIMEOUT, $timeout);
		}

		/* 	if($username != ''){
		  curl_setopt($_session, CURLOPT_USERPWD, $username . ':' . $password);
		  } */

		if(isset($_pathA[1]) && $_pathA[1] != ''){
			$_url_param = explode('&', $_pathA[1]);
			foreach($_url_param as $cur){
				$_param_split = explode('=', $cur);
				$_params[$_param_split[0]] = isset($_param_split[1]) ? $_param_split[1] : '';
			}
		}

		if($files){
			foreach($files as $k => $v){
				$_params[$k] = '@' . $v;
			}
		}

		if($_params){
			curl_setopt($_session, CURLOPT_POST, 1);
			curl_setopt($_session, CURLOPT_POSTFIELDS, $_params);
		}

		if($header){
			curl_setopt($_session, CURLOPT_HEADER, 1);
		}

		if(defined('WE_PROXYHOST') && WE_PROXYHOST != ''){

			$_proxyhost = defined('WE_PROXYHOST') ? WE_PROXYHOST : '';
			$_proxyport = (defined('WE_PROXYPORT') && WE_PROXYPORT) ? WE_PROXYPORT : '80';
			$_proxy_user = defined('WE_PROXYUSER') ? WE_PROXYUSER : '';
			$_proxy_pass = defined('WE_PROXYPASSWORD') ? WE_PROXYPASSWORD : '';

			if($_proxyhost != ''){
				curl_setopt($_session, CURLOPT_PROXY, $_proxyhost . ':' . $_proxyport);
				if($_proxy_user != ''){
					curl_setopt($_session, CURLOPT_PROXYUSERPWD, $_proxy_user . ':' . $_proxy_pass);
				}
				curl_setopt($_session, CURLOPT_SSL_VERIFYPEER, FALSE);
			}
		}

		$_data = curl_exec($_session);

		if(curl_errno($_session)){
			$_response['status'] = 1;
			$_response['error'] = curl_error($_session);
			return false;
		}
		$_response['status'] = 0;
		$_response['data'] = $_data;
		curl_close($_session);


		return $_response;
	}

	function convertDateInRequest(array &$req, $asInt = false){
		$dates = $regs = array();

		foreach($req as $key => $value){
			if(preg_match('/^we_date_([a-zA-Z0-9_]+)_(day|month|year|minute|hour)$/', $key, $regs)){
				$dates[$regs[1]][$regs[2]] = $value;
				unset($req[$key]);
			}
		}
		foreach($dates as $k => $vv){
			if($vv['year'] == 0){
				$vv['month'] = $vv['day'] = $vv['hour'] = $vv['minute'] = 0;
			}
			$req[$k] = ($asInt ?
					mktime($vv['hour'], $vv['minute'], 0, $vv['month'], $vv['day'], $vv['year']) :
					sprintf('%04d-%02d-%02d %02d:%02d:00', $vv['year'], $vv['month'], $vv['day'], $vv['hour'], $vv['minute']));
		}
	}

	/**
	 * This function works in very same way as the standard array_splice function
	 * except the second parametar is the array index and not just offset
	 * The functions modifies the array that has been passed by reference as the first function parametar
	 *
	 * @param          array                                  $a
	 * @param          interger                                $start
	 * @param          integer                                 $len
	 *
	 *
	 * @return         none
	 */
	public static function new_array_splice(&$a, $start, $len = 1){
		$ks = array_keys($a);
		$k = array_search($start, $ks);
		if($k !== false){
			$ks = array_splice($ks, $k, $len);
			foreach($ks as $k){
				unset($a[$k]);
			}
		}
	}

	public static function getMimeType($ext, $filepath = '', $method = self::MIME_BY_HEAD_THEN_EXTENSION){
		switch($filepath ? $method : self::MIME_BY_EXTENSION){
			case self::MIME_BY_DATA:
				if(function_exists('finfo_open')){
					$finfo = finfo_open(FILEINFO_MIME_TYPE);
					$mime = finfo_buffer($finfo, $filepath);
					finfo_close($finfo);
					if($mime){
						return $mime;
					}
				}
				break;
			case self::MIME_BY_HEAD:
			case self::MIME_BY_HEAD_THEN_EXTENSION:
				if(function_exists('finfo_open')){
					$finfo = finfo_open(FILEINFO_MIME_TYPE);
					$mime = finfo_file($finfo, $filepath);
					finfo_close($finfo);
					if($mime || $method == self::MIME_BY_HEAD){
						return $mime ? $mime : false;
					}
				}
				if(function_exists('mime_content_type')){
					$mime = mime_content_type($filepath);
					if($mime || $method == self::MIME_BY_HEAD){
						return $mime ? $mime : false;
					}
				}
				if($method == self::MIME_BY_HEAD){
					return false;
				}
				break;
			case self::MIME_BY_EXTENSION:
				break;
			default:
				//false means: no info about MIME type can be determined
				return false;
		}
		//self::MIME_BY_EXTENSION
		return (!isset(self::$mimetypes[strtolower($ext)])) ? 'application/octet-stream' : self::$mimetypes[strtolower($ext)];
	}

	public static function extension2mime($ext){
		return isset(self::$mimetypes[strtolower($ext)]) ? self::$mimetypes[strtolower($ext)] : false;
	}

	public static function mime2extensions($mime, $retCsv = false){
		$mime = str_replace('/*', '/', trim($mime));
		$extensions = array();
		foreach(self::$mimetypes as $k => $v){
			if(strpos($v, $mime) === 0){
				$extensions[] = $k;
			}
		}

		return $retCsv ? implode(',', $extensions) : $extensions;
	}

	public static function mimegroup2mimes($mimegroup, $retCsv = false){
		$mimegroup = str_replace('/*', '/', trim($mimegroup));
		$mimes = array();
		foreach(self::$mimetypes as $v){
			if(strpos($v, $mimegroup) === 0 && !in_array($v, $mimes)){
				$mimes[] = $v;
			}
		}

		return $retCsv ? implode(',', $mimes) : $mimes;
	}

	public static function isExtensionMime($ext, $mime){
		$mime = str_replace('/*', '/', trim($mime));

		return strpos(self::$mimetypes[$ext], $mime) === false ? false : true;
	}

}
